// This shader computes the chromatic aberration effect

// Since post processing is a fullscreen effect, we use the fullscreen vertex shader provided by bevy.
// This will import a vertex shader that renders a single fullscreen triangle.
//
// A fullscreen triangle is a single triangle that covers the entire screen.
// The box in the top left in that diagram is the screen. The 4 x are the corner of the screen
//
// Y axis
//  1 |  x-----x......
//  0 |  |  s  |  . ´
// -1 |  x_____x´
// -2 |  :  .´
// -3 |  :´
//    +---------------  X axis
//      -1  0  1  2  3
//
// As you can see, the triangle ends up bigger than the screen.
//
// You don't need to worry about this too much since bevy will compute the correct UVs for you.
#import bevy_core_pipeline::fullscreen_vertex_shader::FullscreenVertexOutput

@group(0) @binding(0) var screen_texture: texture_2d<f32>;
@group(0) @binding(1) var texture_sampler: sampler;
struct PostProcessSettings {
    chromab: f32,
    banding: f32,
    t: f32,
    w: f32,
    h: f32,
    x: u32,
    y: u32,
#ifdef SIXTEEN_BYTE_ALIGNMENT
    // WebGL2 structs must be 16 byte aligned.
    _webgl2_padding: vec3<f32>
#endif
}
@group(0) @binding(2) var<uniform> settings: PostProcessSettings;
// TODO do this better - pass in screen size - allow effect strength to vary
@fragment
fn fragment(in: FullscreenVertexOutput) -> @location(0) vec4<f32> {
    // Chromatic aberration strength
    let dx = (in.position.x/settings.w)-0.5;
    let dy = (in.position.y/settings.h)-0.5;

    let offset_strength = abs(dx) * settings.chromab * (dx/abs(dx));

    let y_glitch = (dy-settings.t/2000.0) % 0.4 + 0.2;
    let shift_strength = dy * dx * 0.00005 / (abs(y_glitch) + 0.001) * (y_glitch/abs(y_glitch));

    let banding_a = (y_glitch % 0.01)*2 + 0.98;

    let ltr = in.position.y-in.position.x+settings.w;
    let rtl = in.position.y+in.position.x;
    let banding_r = 1.0 - abs((ltr % 8.0)-4.0) * settings.banding / 4.0;
    let banding_g = 1.0 - abs((rtl % 10.0)-5.0) * settings.banding / 8.0;
    let banding_b = 1.0 - abs((ltr % 9.0)-4.5) * settings.banding;

    // Sample each color channel with an arbitrary shift
    return vec4<f32>(
        textureSample(screen_texture, texture_sampler, in.uv + vec2<f32>(shift_strength, 0.0)).r * banding_r * banding_a,
        textureSample(screen_texture, texture_sampler, in.uv + vec2<f32>(shift_strength-offset_strength, 0.0)).g * banding_g * banding_a,
        textureSample(screen_texture, texture_sampler, in.uv + vec2<f32>(shift_strength-offset_strength*2, 0.0)).b * banding_b * banding_a,
        1.0,
    );
}

